<?php

namespace plugin\wz_chat\app\model;

use support\Model;

/**
 * @property string $id 请求ID
 * @property string $requester_id 申请者用户ID
 * @property string $receiver_id 接收者用户ID
 * @property string|null $message 申请留言
 * @property string $status 状态 pending-待处理 accepted-已通过 rejected-已拒绝 cancelled-已取消
 * @property \Carbon\Carbon $created_at 创建时间
 * @property \Carbon\Carbon|null $updated_at 更新时间
 *
 * @property Users $requester 申请者
 * @property Users $receiver 接收者
 */
class FriendRequests extends Model
{
    protected $table = 'wz_friend_requests';
    protected $primaryKey = 'id'; 
    public $incrementing = true; 
    public $timestamps = true; // 使用 created_at 和 updated_at
    
    
    // 状态常量（与表结构匹配）
    const STATUS_PENDING = 'pending';
    const STATUS_ACCEPTED = 'accepted';
    const STATUS_REJECTED = 'rejected';
    const STATUS_CANCELLED = 'cancelled';
    
    protected $fillable = [
        'id',
        'requester_id',
        'receiver_id',
        'message',
        'status'
    ];
    
    protected $casts = [
        'created_at' => 'datetime',
        'updated_at' => 'datetime'
    ];
    
    /**
     * 关联申请者
     */
    public function requester()
    {
        return $this->belongsTo(Users::class, 'requester_id');
    }
    
    /**
     * 关联接收者
     */
    public function receiver()
    {
        return $this->belongsTo(Users::class, 'receiver_id');
    }
    
    /**
     * 获取状态名称
     */
    public function getStatusNameAttribute()
    {
        $statuses = [
            self::STATUS_PENDING => '待处理',
            self::STATUS_ACCEPTED => '已通过',
            self::STATUS_REJECTED => '已拒绝',
            self::STATUS_CANCELLED => '已取消'
        ];
        return $statuses[$this->status] ?? '未知状态';
    }
    
    /**
     * 判断是否是好友申请
     */
    public function isFriendRequest()
    {
        return true; // 此表专用于好友申请
    }
    
    /**
     * 获取请求方向（相对于当前用户）
     */
    public function getDirectionAttribute($userId)
    {
        if ($this->requester_id === $userId) {
            return 'sent';
        }
        return 'received';
    }
    
    /**
     * 接受申请
     */
    public function accept()
    {
        $this->status = self::STATUS_ACCEPTED;
        return $this->save();
    }
    
    /**
     * 拒绝申请
     */
    public function reject()
    {
        $this->status = self::STATUS_REJECTED;
        return $this->save();
    }
    
    /**
     * 取消申请
     */
    public function cancel()
    {
        $this->status = self::STATUS_CANCELLED;
        return $this->save();
    }
}