class WzPush {
  constructor(url) {
    this.url = url;
    this.socket = null;
    this.listeners = new Map();
    this.reconnectAttempts = 0;
    this.maxReconnectAttempts = 5;
    this.reconnectInterval = 3000; // 3秒重连间隔
    this.heartbeatInterval = 20000; // 20秒心跳间隔
    this.heartbeatTimer = null;
    this.isConnected = false;

    // 初始化连接
    this.connect();
  }

  // 连接WebSocket
  connect() {
    this.socket = new WebSocket(this.url);

    this.socket.onopen = () => {
      console.log('WebSocket connected');
      this.isConnected = true;
      this.reconnectAttempts = 0;
      this.startHeartbeat();
    };

    this.socket.onmessage = (event) => {
      try {
        const message = JSON.parse(event.data);
        this.handleMessage(message);
      } catch (error) {
        console.error('Error parsing message:', error);
      }
    };

    this.socket.onclose = () => {
      console.log('WebSocket disconnected');
      this.isConnected = false;
      this.stopHeartbeat();
      this.scheduleReconnect();
    };

    this.socket.onerror = (error) => {
      console.error('WebSocket error:', error);
    };
  }

  // 处理接收到的消息
  handleMessage(message) {
    if (!message.type) {
      console.warn('Received message without type:', message);
      return;
    }

    // 处理心跳响应
    if (message.type === 'pong') {
      return;
    }

    // 调用对应的监听器
    if (this.listeners.has(message.type)) {
      const callbacks = this.listeners.get(message.type);
      const data = message.data || {};
      callbacks.forEach(callback => callback(data));
    } else {
      console.log(`No listener for message type: ${message.type}`);
    }
  }

  // 添加消息监听器
  on(type, callback) {
    if (typeof callback !== 'function') {
      throw new Error('Callback must be a function');
    }

    if (!this.listeners.has(type)) {
      this.listeners.set(type, []);
    }
    this.listeners.get(type).push(callback);
    return this;
  }

  // 移除消息监听器
  off(type, callback) {
    if (this.listeners.has(type)) {
      const callbacks = this.listeners.get(type);
      const index = callbacks.indexOf(callback);
      if (index !== -1) {
        callbacks.splice(index, 1);
      }
    }
    return this;
  }

  // 发送消息
  send(type, data = {}) {
    if (!this.isConnected) {
      console.warn('Cannot send message, WebSocket is not connected');
      return false;
    }

    try {
      const message = JSON.stringify({ type, data });
      this.socket.send(message);
      return true;
    } catch (error) {
      console.error('Error sending message:', error);
      return false;
    }
  }

  // 开始心跳检测
  startHeartbeat() {
    this.stopHeartbeat();
    this.heartbeatTimer = setInterval(() => {
      if (this.isConnected) {
        this.send('ping');
      }
    }, this.heartbeatInterval);
  }

  // 停止心跳检测
  stopHeartbeat() {
    if (this.heartbeatTimer) {
      clearInterval(this.heartbeatTimer);
      this.heartbeatTimer = null;
    }
  }

  // 安排重连
  scheduleReconnect() {
    if (this.reconnectAttempts < this.maxReconnectAttempts) {
      this.reconnectAttempts++;
      console.log(`Attempting to reconnect (${this.reconnectAttempts}/${this.maxReconnectAttempts})...`);
      setTimeout(() => this.connect(), this.reconnectInterval);
    } else {
      console.log('Max reconnection attempts reached');
    }
  }

  // 关闭连接
  close() {
    this.stopHeartbeat();
    this.isConnected = false;
    this.socket.close();
  }
}

// 导出模块
if (typeof module !== 'undefined' && module.exports) {
  module.exports = WzPush;
} else if (typeof define === 'function' && define.amd) {
  define([], () => WzPush);
} else {
  window.WzPush = WzPush;
}